@@ -179,6 +179,32 @@ def stringify() -> None:
         stringify_file(path)
 # }}}
 
+# {{{ Bitfields
+
+def make_bitfields() -> None:
+    from kitty.fast_data_types import SCALE_BITS, SUBSCALE_BITS, WIDTH_BITS
+
+    from .bitfields import make_bitfield
+
+    def mb(*args: str) -> None:
+        output_path, ans = make_bitfield(*args)
+        with replace_if_needed(output_path) as buf:
+            print(ans, file=buf)
+
+    mb(
+        'tools/vt', 'CellAttrs',
+        'decoration 3', 'bold 1', 'italic 1', 'reverse 1', 'strike 1', 'dim 1', 'hyperlink_id 16',
+    )
+    mb('tools/vt', 'Ch', 'is_idx 1', 'ch_or_idx 31')
+    mb(
+        'tools/vt', 'MultiCell',
+        'is_multicell 1', 'natural_width 1', f'scale {SCALE_BITS}', f'subscale_n {SUBSCALE_BITS}', f'subscale_d {SUBSCALE_BITS}',
+        f'width {WIDTH_BITS}', f'x {WIDTH_BITS + SCALE_BITS + 1}', f'y {SCALE_BITS + 1}', 'vertical_align 3',
+    )
+    mb('tools/vt', 'CellColor', 'is_idx 1', 'red 8', 'green 8', 'blue 8')
+    mb('tools/vt', 'LineAttrs', 'prompt_kind 2',)
+# }}}
+
 # Completions {{{
 
 @lru_cache
@@ -906,6 +932,7 @@ def main(args: list[str]=sys.argv) -> None:
     update_at_commands()
     kitten_clis()
     stringify()
+    make_bitfields()
     print(json.dumps(changed, indent=2))
     stdout, stderr = simdgen_process.communicate()
     if simdgen_process.wait() != 0:
