@@ -46,7 +46,7 @@ func (self *Input) has_mime_matching(predicate func(string) bool) bool {
 }
 
 func write_loop(inputs []*Input, opts *Options) (err error) {
-	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.NoMouseTracking)
+	lp, err := loop.New(loop.NoAlternateScreen, loop.NoRestoreColors, loop.NoMouseTracking, loop.NoInBandResizeNotifications)
 	if err != nil {
 		return err
 	}
@@ -222,8 +222,6 @@ func run_set_loop(opts *Options, args []string) (err error) {
 			return fmt.Errorf("Could not guess MIME type for %s use the --mime option to specify a MIME type", arg)
 		}
 		to_process[i] = inputs[i]
-		if to_process[i].is_stream {
-		}
 	}
 	return write_loop(to_process, opts)
 }
