@@ -61,6 +61,7 @@ type Handler struct {
 	collection                                          *Collection
 	diff_map                                            map[string]*Patch
 	logical_lines                                       *LogicalLines
+	terminal_capabilities_received                      bool
 	lp                                                  *loop.Loop
 	current_context_count, original_context_count       int
 	added_count, removed_count                          int
@@ -111,9 +112,55 @@ func (self *Handler) finalize() {
 	image_collection.Finalize(self.lp)
 }
 
+func set_terminal_colors(lp *loop.Loop) {
+	create_formatters()
+	lp.SetDefaultColor(loop.FOREGROUND, resolved_colors.Foreground)
+	lp.SetDefaultColor(loop.CURSOR, resolved_colors.Foreground)
+	lp.SetDefaultColor(loop.BACKGROUND, resolved_colors.Background)
+	lp.SetDefaultColor(loop.SELECTION_BG, resolved_colors.Select_bg)
+	if resolved_colors.Select_fg.IsSet {
+		lp.SetDefaultColor(loop.SELECTION_FG, resolved_colors.Select_fg.Color)
+	}
+}
+
+func (self *Handler) on_capabilities_received(tc loop.TerminalCapabilities) {
+	var use_dark_colors bool
+	prev := use_light_colors
+	switch conf.Color_scheme {
+	case Color_scheme_auto:
+		use_dark_colors = tc.ColorPreference != loop.LIGHT_COLOR_PREFERENCE
+	case Color_scheme_light:
+		use_dark_colors = false
+	case Color_scheme_dark:
+		use_dark_colors = true
+	}
+	use_light_colors = !use_dark_colors
+	if use_light_colors != prev && (light_highlight_started || dark_highlight_started) {
+		self.highlight_all()
+	}
+	set_terminal_colors(self.lp)
+	self.terminal_capabilities_received = true
+	self.draw_screen()
+}
+
+func (self *Handler) on_color_scheme_change(cp loop.ColorPreference) error {
+	if conf.Color_scheme != Color_scheme_auto {
+		return nil
+	}
+	light := cp == loop.LIGHT_COLOR_PREFERENCE
+	if use_light_colors != light {
+		use_light_colors = light
+		set_terminal_colors(self.lp)
+		self.highlight_all()
+		self.draw_screen()
+	}
+	return nil
+}
+
 func (self *Handler) initialize() {
 	self.rl = readline.New(self.lp, readline.RlInit{DontMarkPrompts: true, Prompt: "/"})
 	self.lp.OnEscapeCode = self.on_escape_code
+	self.lp.OnColorSchemeChange = self.on_color_scheme_change
 	image_collection = graphics.NewImageCollection()
 	self.current_context_count = opts.Context
 	if self.current_context_count < 0 {
@@ -122,13 +169,6 @@ func (self *Handler) initialize() {
 	sz, _ := self.lp.ScreenSize()
 	self.update_screen_size(sz)
 	self.original_context_count = self.current_context_count
-	self.lp.SetDefaultColor(loop.FOREGROUND, conf.Foreground)
-	self.lp.SetDefaultColor(loop.CURSOR, conf.Foreground)
-	self.lp.SetDefaultColor(loop.BACKGROUND, conf.Background)
-	self.lp.SetDefaultColor(loop.SELECTION_BG, conf.Select_bg)
-	if conf.Select_fg.IsSet {
-		self.lp.SetDefaultColor(loop.SELECTION_FG, conf.Select_fg.Color)
-	}
 	self.async_results = make(chan AsyncResult, 32)
 	go func() {
 		r := AsyncResult{}
@@ -176,15 +216,25 @@ func (self *Handler) on_wakeup() error {
 	}
 }
 
+var dark_highlight_started bool
+var light_highlight_started bool
+
 func (self *Handler) highlight_all() {
+	if (use_light_colors && light_highlight_started) || (!use_light_colors && dark_highlight_started) {
+		return
+	}
+	if use_light_colors {
+		light_highlight_started = true
+	} else {
+		dark_highlight_started = true
+	}
 	text_files := utils.Filter(self.collection.paths_to_highlight.AsSlice(), is_path_text)
 	go func() {
 		r := AsyncResult{rtype: HIGHLIGHT}
-		highlight_all(text_files)
+		highlight_all(text_files, use_light_colors)
 		self.async_results <- r
 		self.lp.WakeupMainThread()
 	}()
-
 }
 
 func (self *Handler) load_all_images() {
@@ -316,7 +366,7 @@ func (self *Handler) render_diff() (err error) {
 	return nil
 }
 
-func (self *Handler) draw_image(key string, num_rows, starting_row int) {
+func (self *Handler) draw_image(key string, _, starting_row int) {
 	image_collection.PlaceImageSubRect(self.lp, key, self.images_resized_to, 0, self.screen_size.cell_height*starting_row, -1, -1)
 }
 
@@ -346,7 +396,7 @@ func (self *Handler) draw_screen() {
 	}
 	lp.MoveCursorTo(1, 1)
 	lp.ClearToEndOfScreen()
-	if self.logical_lines == nil || self.diff_map == nil || self.collection == nil {
+	if self.logical_lines == nil || self.diff_map == nil || self.collection == nil || !self.terminal_capabilities_received {
 		lp.Println(`Calculating diff, please wait...`)
 		return
 	}
@@ -523,6 +573,27 @@ func (self *Handler) scroll_to_next_change(backwards bool) bool {
 	return false
 }
 
+func (self *Handler) scroll_to_next_file(backwards bool) bool {
+	if backwards {
+		for i := self.scroll_pos.logical_line - 1; i >= 0; i-- {
+			line := self.logical_lines.At(i)
+			if line.line_type == TITLE_LINE {
+				self.scroll_pos = ScrollPos{i, 0}
+				return true
+			}
+		}
+	} else {
+		for i := self.scroll_pos.logical_line + 1; i < self.logical_lines.Len(); i++ {
+			line := self.logical_lines.At(i)
+			if line.line_type == TITLE_LINE {
+				self.scroll_pos = ScrollPos{i, 0}
+				return true
+			}
+		}
+	}
+	return false
+}
+
 func (self *Handler) scroll_to_next_match(backwards, include_current_match bool) bool {
 	if self.current_search == nil {
 		return false
@@ -616,6 +687,8 @@ func (self *Handler) dispatch_action(name, args string) error {
 	case `scroll_to`:
 		done := false
 		switch {
+		case strings.Contains(args, "file"):
+			done = self.scroll_to_next_file(strings.Contains(args, `prev`))
 		case strings.Contains(args, `change`):
 			done = self.scroll_to_next_change(strings.Contains(args, `prev`))
 		case strings.Contains(args, `match`):
