@@ -1,5 +1,4 @@
-#!/usr/bin/env python3
-# vim:fileencoding=utf-8
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 import asyncio
@@ -11,25 +10,20 @@ import selectors
 import signal
 import sys
 import termios
-from contextlib import contextmanager
+from collections.abc import Callable, Generator
+from contextlib import contextmanager, suppress
+from enum import Enum, IntFlag, auto
 from functools import partial
-from typing import Any, Callable, Dict, Generator, List, NamedTuple, Optional
+from typing import Any, NamedTuple
 
 from kitty.constants import is_macos
-from kitty.fast_data_types import (
-    close_tty, normal_tty, open_tty, parse_input_from_terminal, raw_tty
-)
-from kitty.key_encoding import (
-    ALT, CTRL, PRESS, RELEASE, REPEAT, SHIFT, backspace_key, decode_key_event,
-    enter_key, key_defs as K
-)
+from kitty.fast_data_types import FILE_TRANSFER_CODE, close_tty, normal_tty, open_tty, parse_input_from_terminal, raw_tty
+from kitty.key_encoding import ALT, CTRL, SHIFT, backspace_key, decode_key_event, enter_key
 from kitty.typing import ImageManagerType, KeyEventType, Protocol
-from kitty.utils import ScreenSizeGetter, screen_size_function, write_all
+from kitty.utils import ScreenSize, ScreenSizeGetter, screen_size_function, write_all
 
 from .handler import Handler
-from .operations import init_state, reset_state
-
-C, D = K['C'], K['D']
+from .operations import MouseTracking, init_state, reset_state
 
 
 class BinaryWrite(Protocol):
@@ -41,41 +35,54 @@ class BinaryWrite(Protocol):
         pass
 
 
+def debug_write(*a: Any, **kw: Any) -> None:
+    from base64 import standard_b64encode
+    fobj = kw.pop('file', sys.stderr.buffer)
+    buf = io.StringIO()
+    kw['file'] = buf
+    print(*a, **kw)
+    stext = buf.getvalue()
+    for i in range(0, len(stext), 256):
+        chunk = stext[i:i + 256]
+        text = b'\x1bP@kitty-print|' + standard_b64encode(chunk.encode('utf-8')) + b'\x1b\\'
+        fobj.write(text)
+    fobj.flush()
+
+
 class Debug:
 
-    fobj: Optional[BinaryWrite] = None
+    fobj: BinaryWrite | None = None
 
     def __call__(self, *a: Any, **kw: Any) -> None:
-        from base64 import standard_b64encode
-        buf = io.StringIO()
-        kw['file'] = buf
-        print(*a, **kw)
-        stext = buf.getvalue()
-        text = b'\x1bP@kitty-print|' + standard_b64encode(stext.encode('utf-8')) + b'\x1b\\'
-        fobj = self.fobj or sys.stdout.buffer
-        fobj.write(text)
-        fobj.flush()
+        kw['file'] = self.fobj or sys.stdout.buffer
+        debug_write(*a, **kw)
 
 
 debug = Debug()
+ftc_code = str(FILE_TRANSFER_CODE)
 
 
 class TermManager:
 
-    def __init__(self, optional_actions: int = termios.TCSANOW) -> None:
-        self.extra_finalize: Optional[str] = None
+    def __init__(
+        self, optional_actions: int = termios.TCSANOW, use_alternate_screen: bool = True,
+        mouse_tracking: MouseTracking = MouseTracking.none
+    ) -> None:
+        self.extra_finalize: str | None = None
         self.optional_actions = optional_actions
+        self.use_alternate_screen = use_alternate_screen
+        self.mouse_tracking = mouse_tracking
 
     def set_state_for_loop(self, set_raw: bool = True) -> None:
         if set_raw:
             raw_tty(self.tty_fd, self.original_termios)
-        write_all(self.tty_fd, init_state())
+        write_all(self.tty_fd, init_state(self.use_alternate_screen, self.mouse_tracking))
 
     def reset_state_to_original(self) -> None:
         normal_tty(self.tty_fd, self.original_termios)
         if self.extra_finalize:
             write_all(self.tty_fd, self.extra_finalize)
-        write_all(self.tty_fd, reset_state())
+        write_all(self.tty_fd, reset_state(self.use_alternate_screen))
 
     @contextmanager
     def suspend(self) -> Generator['TermManager', None, None]:
@@ -89,41 +96,60 @@ class TermManager:
         return self
 
     def __exit__(self, *a: object) -> None:
-        self.reset_state_to_original()
-        close_tty(self.tty_fd, self.original_termios)
-        del self.tty_fd, self.original_termios
+        with suppress(Exception):
+            self.reset_state_to_original()
+            close_tty(self.tty_fd, self.original_termios)
+            del self.tty_fd, self.original_termios
 
 
-LEFT, MIDDLE, RIGHT, FOURTH, FIFTH = 1, 2, 4, 8, 16
-DRAG = REPEAT
-bmap = {0: LEFT, 1: MIDDLE, 2: RIGHT}
-MOTION_INDICATOR = 1 << 5
-EXTRA_BUTTON_INDICATOR = 1 << 6
+class MouseButton(IntFlag):
+    NONE, LEFT, MIDDLE, RIGHT, FOURTH, FIFTH, SIXTH, SEVENTH = 0, 1, 2, 4, 8, 16, 32, 64
+    WHEEL_UP, WHEEL_DOWN, WHEEL_LEFT, WHEEL_RIGHT = -1, -2, -4, -8
+
+
+bmap = MouseButton.LEFT, MouseButton.MIDDLE, MouseButton.RIGHT
+ebmap = MouseButton.FOURTH, MouseButton.FIFTH, MouseButton.SIXTH, MouseButton.SEVENTH
+wbmap = MouseButton.WHEEL_UP, MouseButton.WHEEL_DOWN, MouseButton.WHEEL_LEFT, MouseButton.WHEEL_RIGHT
 SHIFT_INDICATOR = 1 << 2
 ALT_INDICATOR = 1 << 3
 CTRL_INDICATOR = 1 << 4
+MOTION_INDICATOR = 1 << 5
+
+
+class EventType(Enum):
+    PRESS = auto()
+    RELEASE = auto()
+    MOVE = auto()
 
 
 class MouseEvent(NamedTuple):
-    x: int
-    y: int
-    type: int
-    buttons: int
+    cell_x: int
+    cell_y: int
+    pixel_x: int
+    pixel_y: int
+    type: EventType
+    buttons: MouseButton
     mods: int
 
 
-def decode_sgr_mouse(text: str) -> MouseEvent:
+def pixel_to_cell(px: int, length: int, cell_length: int) -> int:
+    px = max(0, min(px, length - 1))
+    return px // cell_length
+
+
+def decode_sgr_mouse(text: str, screen_size: ScreenSize) -> MouseEvent:
     cb_, x_, y_ = text.split(';')
     m, y_ = y_[-1], y_[:-1]
     cb, x, y = map(int, (cb_, x_, y_))
-    typ = RELEASE if m == 'm' else (DRAG if cb & MOTION_INDICATOR else PRESS)
-    buttons = 0
+    typ = EventType.RELEASE if m == 'm' else (EventType.MOVE if cb & MOTION_INDICATOR else EventType.PRESS)
+    buttons: MouseButton = MouseButton.NONE
     cb3 = cb & 3
-    if cb3 != 3:
-        if cb & EXTRA_BUTTON_INDICATOR:
-            buttons |= FIFTH if cb3 & 1 else FOURTH
-        else:
-            buttons |= bmap[cb3]
+    if cb >= 128:
+        buttons |= ebmap[cb3]
+    elif cb >= 64:
+        buttons |= wbmap[cb3]
+    elif cb3 < 3:
+        buttons |= bmap[cb3]
     mods = 0
     if cb & SHIFT_INDICATOR:
         mods |= SHIFT
@@ -131,7 +157,10 @@ def decode_sgr_mouse(text: str) -> MouseEvent:
         mods |= ALT
     if cb & CTRL_INDICATOR:
         mods |= CTRL
-    return MouseEvent(x, y, typ, buttons, mods)
+    return MouseEvent(
+        pixel_to_cell(x, screen_size.width, screen_size.cell_width), pixel_to_cell(y, screen_size.height, screen_size.cell_height),
+        x, y, typ, buttons, mods
+    )
 
 
 class UnhandledException(Handler):
@@ -146,10 +175,10 @@ class UnhandledException(Handler):
         self.cmd.set_default_colors()
         self.write(self.tb.replace('\n', '\r\n'))
         self.write('\r\n')
-        self.write('Press the Enter key to quit')
+        self.write('Press Enter to quit')
 
     def on_key(self, key_event: KeyEventType) -> None:
-        if key_event is enter_key:
+        if key_event.key == 'ENTER':
             self.quit_loop(1)
 
     def on_interrupt(self) -> None:
@@ -162,23 +191,24 @@ class SignalManager:
     def __init__(
         self,
         loop: asyncio.AbstractEventLoop,
-        on_winch: Callable,
-        on_interrupt: Callable,
-        on_term: Callable
+        on_winch: Callable[[], None],
+        on_interrupt: Callable[[], None],
+        on_term: Callable[[], None],
+        on_hup: Callable[[], None],
     ) -> None:
-        self.asycio_loop = loop
+        self.asyncio_loop = loop
         self.on_winch, self.on_interrupt, self.on_term = on_winch, on_interrupt, on_term
+        self.on_hup = on_hup
 
     def __enter__(self) -> None:
-        tuple(map(lambda x: self.asycio_loop.add_signal_handler(*x), (
-            (signal.SIGWINCH, self.on_winch),
-            (signal.SIGINT, self.on_interrupt),
-            (signal.SIGTERM, self.on_term)
-        )))
+        self.asyncio_loop.add_signal_handler(signal.SIGWINCH, self.on_winch)
+        self.asyncio_loop.add_signal_handler(signal.SIGINT, self.on_interrupt)
+        self.asyncio_loop.add_signal_handler(signal.SIGTERM, self.on_term)
+        self.asyncio_loop.add_signal_handler(signal.SIGHUP, self.on_hup)
 
     def __exit__(self, *a: Any) -> None:
-        tuple(map(self.asycio_loop.remove_signal_handler, (
-            signal.SIGWINCH, signal.SIGINT, signal.SIGTERM)))
+        tuple(map(self.asyncio_loop.remove_signal_handler, (
+            signal.SIGWINCH, signal.SIGINT, signal.SIGTERM, signal.SIGHUP)))
 
 
 sanitize_bracketed_paste: str = '[\x03\x04\x0e\x0f\r\x07\x7f\x8d\x8e\x8f\x90\x9b\x9d\x9e\x9f]'
@@ -194,11 +224,12 @@ class Loop:
         if is_macos:
             # On macOS PTY devices are not supported by the KqueueSelector and
             # the PollSelector is broken, causes 100% CPU usage
-            self.asycio_loop: asyncio.AbstractEventLoop = asyncio.SelectorEventLoop(selectors.SelectSelector())
-            asyncio.set_event_loop(self.asycio_loop)
+            self.asyncio_loop: asyncio.AbstractEventLoop = asyncio.SelectorEventLoop(selectors.SelectSelector())
+            asyncio.set_event_loop(self.asyncio_loop)
         else:
-            self.asycio_loop = asyncio.get_event_loop()
+            self.asyncio_loop = asyncio.get_event_loop()
         self.return_code = 0
+        self.overlay_ready_reported = False
         self.optional_actions = optional_actions
         self.read_buf = ''
         self.decoder = codecs.getincrementaldecoder('utf-8')('ignore')
@@ -219,7 +250,9 @@ class Loop:
         except BlockingIOError:
             return
         if not bdata:
-            raise EOFError('The input stream is closed')
+            handler.terminal_io_ended = True
+            self.quit(1)
+            return
         data = self.decoder.decode(bdata)
         if self.read_buf:
             data = self.read_buf + data
@@ -274,70 +307,81 @@ class Loop:
             if csi.startswith('<'):
                 # SGR mouse event
                 try:
-                    ev = decode_sgr_mouse(csi[1:])
+                    ev = decode_sgr_mouse(csi[1:], self.handler.screen_size)
                 except Exception:
                     pass
                 else:
-                    self.handler.on_mouse(ev)
-        elif q == '~':
+                    self.handler.on_mouse_event(ev)
+        elif q in 'u~ABCDEHFPQRS':
             if csi == '200~':
                 self.in_bracketed_paste = True
+                return
             elif csi == '201~':
                 self.in_bracketed_paste = False
+                return
+            try:
+                k = decode_key_event(csi[:-1], q)
+            except Exception:
+                pass
+            else:
+                if not self.handler.perform_default_key_action(k):
+                    self.handler.on_key_event(k)
 
     def _on_pm(self, pm: str) -> None:
         pass
 
     def _on_osc(self, osc: str) -> None:
-        m = re.match(r'(\d+);', osc)
-        if m is not None:
-            code = int(m.group(1))
-            rest = osc[m.end():]
-            if code == 52:
-                where, rest = rest.partition(';')[::2]
-                from_primary = 'p' in where
+        idx = osc.find(';')
+        if idx <= 0:
+            return
+        q = osc[:idx]
+        if q == '52':
+            widx = osc.find(';', idx + 1)
+            if widx < idx:
+                from_primary = osc.find('p', idx + 1) > -1
+                payload = ''
+            else:
                 from base64 import standard_b64decode
-                self.handler.on_clipboard_response(standard_b64decode(rest).decode('utf-8'), from_primary)
+                from_primary = osc.find('p', idx+1, widx) > -1
+                data = memoryview(osc.encode('ascii'))
+                payload = standard_b64decode(data[widx+1:]).decode('utf-8')
+            self.handler.on_clipboard_response(payload, from_primary)
+        elif q == ftc_code:
+            from kitty.file_transmission import FileTransmissionCommand
+            data = memoryview(osc.encode('ascii'))
+            self.handler.on_file_transfer_response(FileTransmissionCommand.deserialize(data[idx+1:]))
 
     def _on_apc(self, apc: str) -> None:
-        if apc.startswith('K'):
-            try:
-                k = decode_key_event(apc[1:])
-            except Exception:
-                pass
-            else:
-                if k.mods is CTRL and k.type is not RELEASE:
-                    if k.key is C:
-                        self.handler.on_interrupt()
-                        return
-                    if k.key is D:
-                        self.handler.on_eot()
-                        return
-                self.handler.on_key(k)
-        elif apc.startswith('G'):
+        if apc.startswith('G'):
             if self.handler.image_manager is not None:
                 self.handler.image_manager.handle_response(apc)
     # }}}
 
+    @property
+    def total_pending_bytes_to_write(self) -> int:
+        return sum(map(len, self.write_buf))
+
     def _write_ready(self, handler: Handler, fd: int) -> None:
         if len(self.write_buf) > self.iov_limit:
             self.write_buf[self.iov_limit - 1] = b''.join(self.write_buf[self.iov_limit - 1:])
             del self.write_buf[self.iov_limit:]
-        sizes = tuple(map(len, self.write_buf))
-        total_size = sum(sizes)
+        total_size = self.total_pending_bytes_to_write
         if total_size:
             try:
                 written = os.writev(fd, self.write_buf)
             except BlockingIOError:
                 return
             if not written:
-                raise EOFError('The output stream is closed')
+                handler.terminal_io_ended = True
+                self.quit(1)
+                return
         else:
             written = 0
         if written >= total_size:
-            self.write_buf: List[bytes] = []
-            self.asycio_loop.remove_writer(fd)
+            self.write_buf: list[bytes] = []
+            self.asyncio_loop.remove_writer(fd)
             self.waiting_for_writes = False
+            handler.on_writing_finished()
         else:
             consumed = 0
             for i, buf in enumerate(self.write_buf):
@@ -351,12 +395,12 @@ class Loop:
                 break
             del self.write_buf[:consumed]
 
-    def quit(self, return_code: Optional[int] = None) -> None:
+    def quit(self, return_code: int | None = None) -> None:
         if return_code is not None:
             self.return_code = return_code
-        self.asycio_loop.stop()
+        self.asyncio_loop.stop()
 
-    def loop_impl(self, handler: Handler, term_manager: TermManager, image_manager: Optional[ImageManagerType] = None) -> Optional[str]:
+    def loop_impl(self, handler: Handler, term_manager: TermManager, image_manager: ImageManagerType | None = None) -> str | None:
         self.write_buf = []
         tty_fd = term_manager.tty_fd
         tb = None
@@ -365,10 +409,10 @@ class Loop:
         def schedule_write(data: bytes) -> None:
             self.write_buf.append(data)
             if not self.waiting_for_writes:
-                self.asycio_loop.add_writer(tty_fd, self._write_ready, handler, tty_fd)
+                self.asyncio_loop.add_writer(tty_fd, self._write_ready, handler, tty_fd)
                 self.waiting_for_writes = True
 
-        def handle_exception(loop: asyncio.AbstractEventLoop, context: Dict[str, Any]) -> None:
+        def handle_exception(loop: asyncio.AbstractEventLoop, context: dict[str, Any]) -> None:
             nonlocal tb
             loop.stop()
             tb = context['message']
@@ -377,29 +421,31 @@ class Loop:
                 import traceback
                 tb += '\n' + ''.join(traceback.format_exception(exc.__class__, exc, exc.__traceback__))
 
-        self.asycio_loop.set_exception_handler(handle_exception)
+        self.asyncio_loop.set_exception_handler(handle_exception)
         handler._initialize(self._get_screen_size(), term_manager, schedule_write, self, debug, image_manager)
         with handler:
-            self.asycio_loop.add_reader(
+            if handler.overlay_ready_report_needed:
+                handler.cmd.overlay_ready()
+            self.asyncio_loop.add_reader(
                     tty_fd, self._read_ready, handler, tty_fd)
-            self.asycio_loop.add_writer(
+            self.asyncio_loop.add_writer(
                     tty_fd, self._write_ready, handler, tty_fd)
-            self.asycio_loop.run_forever()
-            self.asycio_loop.remove_reader(tty_fd)
+            self.asyncio_loop.run_forever()
+            self.asyncio_loop.remove_reader(tty_fd)
             if self.waiting_for_writes:
-                self.asycio_loop.remove_writer(tty_fd)
+                self.asyncio_loop.remove_writer(tty_fd)
         return tb
 
     def loop(self, handler: Handler) -> None:
-        tb: Optional[str] = None
+        tb: str | None = None
 
         def _on_sigwinch() -> None:
             self._get_screen_size.changed = True
             handler.screen_size = self._get_screen_size()
             handler.on_resize(handler.screen_size)
 
-        signal_manager = SignalManager(self.asycio_loop, _on_sigwinch, handler.on_interrupt, handler.on_term)
-        with TermManager(self.optional_actions) as term_manager, signal_manager:
+        signal_manager = SignalManager(self.asyncio_loop, _on_sigwinch, handler.on_interrupt, handler.on_term, handler.on_hup)
+        with TermManager(self.optional_actions, handler.use_alternate_screen, handler.mouse_tracking) as term_manager, signal_manager:
             self._get_screen_size: ScreenSizeGetter = screen_size_function(term_manager.tty_fd)
             image_manager = None
             if handler.image_manager_class is not None:
@@ -412,8 +458,12 @@ class Loop:
 
             term_manager.extra_finalize = b''.join(self.write_buf).decode('utf-8')
             if tb is not None:
+                report_overlay_ready = handler.overlay_ready_report_needed and not self.overlay_ready_reported
                 self.return_code = 1
-                self._report_error_loop(tb, term_manager)
+                if not handler.terminal_io_ended:
+                    self._report_error_loop(tb, term_manager, report_overlay_ready)
 
-    def _report_error_loop(self, tb: str, term_manager: TermManager) -> None:
-        self.loop_impl(UnhandledException(tb), term_manager)
+    def _report_error_loop(self, tb: str, term_manager: TermManager, overlay_ready_report_needed: bool) -> None:
+        handler = UnhandledException(tb)
+        handler.overlay_ready_report_needed = overlay_ready_report_needed
+        self.loop_impl(handler, term_manager)
