@@ -438,7 +438,7 @@ ident_in_list_of_notifications(NSString *ident, NSArray<UNNotification*> *list)
 
 void
 cocoa_report_live_notifications(const char* ident) {
-    do_notification_callback(ident, "live", "");
+    do_notification_callback(ident, "live", ident ? ident : "");
 }
 
 static bool
@@ -1202,6 +1202,112 @@ play_system_sound_by_id_async(PyObject *self UNUSED, PyObject *which) {
     Py_RETURN_NONE;
 }
 
+// Dock Progress bar {{{
+@interface RoundedRectangleView : NSView {
+    unsigned intermediate_step;
+    CGFloat fill_fraction;
+    BOOL is_indeterminate;
+}
+- (void) animate;
+- (BOOL) isIndeterminate;
+- (void) setIndeterminate:(BOOL)val;
+- (void) setFraction:(CGFloat) fraction;
+@end
+
+@implementation RoundedRectangleView
+
+- (void) animate { intermediate_step++; }
+- (BOOL) isIndeterminate { return is_indeterminate; }
+- (void) setIndeterminate:(BOOL)val {
+    if (val != is_indeterminate) {
+        is_indeterminate = val;
+        intermediate_step = 0;
+        }
+    }
+- (void) setFraction:(CGFloat)fraction { fill_fraction = fraction; }
+
+
+- (void)drawRect:(NSRect)dirtyRect {
+    [super drawRect:dirtyRect];
+
+    NSRect bar = NSInsetRect(self.bounds, 4, 4);
+    CGFloat cornerRadius = self.bounds.size.height / 4.0;
+
+#define fill(bar) [[NSBezierPath bezierPathWithRoundedRect:bar xRadius:cornerRadius yRadius:cornerRadius] fill]
+    // Create the border
+    [[[NSColor whiteColor] colorWithAlphaComponent:0.8] setFill];
+    fill(bar);
+    // Create the background
+    [[[NSColor blackColor] colorWithAlphaComponent:0.8] setFill];
+    fill(NSInsetRect(bar, 0.5, 0.5));
+    // Create the progress
+    NSRect bar_progress = NSInsetRect(bar, 1, 1);
+    if (intermediate_step) {
+        unsigned num_of_steps = 80;
+        intermediate_step = intermediate_step % num_of_steps;
+        bar_progress.size.width = self.bounds.size.width / 8;
+        float frac = intermediate_step / (float)num_of_steps;
+        bar_progress.origin.x += (self.bounds.size.width - bar_progress.size.width) * frac;
+    } else bar_progress.size.width *= fill_fraction;
+    [[NSColor whiteColor] setFill];
+    fill(bar_progress);
+#undef fill
+}
+
+@end
+static NSView *dock_content_view = nil;
+static NSImageView *dock_image_view = nil;
+static RoundedRectangleView *dock_pbar = nil;
+
+static void
+animate_dock_progress_bar(id_type timer_id UNUSED, void *data UNUSED);
+
+static void
+tick_dock_pbar(void) {
+    add_main_loop_timer(ms_to_monotonic_t(20), false, animate_dock_progress_bar, NULL, NULL);
+}
+
+static void
+animate_dock_progress_bar(id_type timer_id UNUSED, void *data UNUSED) {
+    if (dock_pbar != nil && [dock_pbar isIndeterminate]) {
+        [dock_pbar animate];
+        NSDockTile *dockTile = [NSApp dockTile];
+        [dockTile display];
+        tick_dock_pbar();
+    }
+}
+
+static PyObject*
+cocoa_show_progress_bar_on_dock_icon(PyObject *self UNUSED, PyObject *args) {
+    float percent = -100;
+    if (!PyArg_ParseTuple(args, "|f", &percent)) return NULL;
+    NSDockTile *dockTile = [NSApp dockTile];
+    if (!dock_content_view) {
+        dock_content_view = [[NSView alloc] initWithFrame:NSMakeRect(0, 0, dockTile.size.width, dockTile.size.height)];
+        dock_image_view = [NSImageView.alloc initWithFrame:dock_content_view.frame];
+        dock_image_view.imageScaling = NSImageScaleProportionallyDown;
+        dock_image_view.image = NSApp.applicationIconImage;
+        [dock_content_view addSubview:dock_image_view];
+        dock_pbar = [[RoundedRectangleView alloc] initWithFrame:NSMakeRect(0, 0, dockTile.size.width, dockTile.size.height / 4)];
+        [dock_content_view addSubview:dock_pbar];
+    }
+    [dock_content_view setFrameSize:dockTile.size];
+    [dock_image_view setFrameSize:dockTile.size];
+    if (percent >= 0 && percent <= 100) {
+        [dock_pbar setFraction:percent/100.];
+        [dock_pbar setIndeterminate:NO];
+    } else if (percent > 100) {
+        [dock_pbar setIndeterminate:YES];
+        tick_dock_pbar();
+    }
+    [dock_pbar setFrameSize:NSMakeSize(dockTile.size.width - 20, 20)];
+    [dock_pbar setFrameOrigin:NSMakePoint(10, -2)];
+    [dockTile setContentView:percent < 0 ? nil : dock_content_view];
+    [dockTile display];
+    Py_RETURN_NONE;
+}
+// }}}
+
 static PyMethodDef module_methods[] = {
     {"cocoa_play_system_sound_by_id_async", play_system_sound_by_id_async, METH_O, ""},
     {"cocoa_get_lang", (PyCFunction)cocoa_get_lang, METH_NOARGS, ""},
@@ -1213,6 +1319,7 @@ static PyMethodDef module_methods[] = {
     {"cocoa_set_url_handler", (PyCFunction)cocoa_set_url_handler, METH_VARARGS, ""},
     {"cocoa_set_app_icon", (PyCFunction)cocoa_set_app_icon, METH_VARARGS, ""},
     {"cocoa_set_dock_icon", (PyCFunction)cocoa_set_dock_icon, METH_VARARGS, ""},
+    {"cocoa_show_progress_bar_on_dock_icon", (PyCFunction)cocoa_show_progress_bar_on_dock_icon, METH_VARARGS, ""},
     {"cocoa_bundle_image_as_png", (PyCFunction)(void(*)(void))bundle_image_as_png, METH_VARARGS | METH_KEYWORDS, ""},
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
