@@ -6,13 +6,14 @@ import inspect
 import os
 import re
 import textwrap
-from typing import Any, Callable, Dict, Iterator, List, Set, Tuple, Union, get_type_hints
+from collections.abc import Callable, Iterator
+from typing import Any, get_type_hints
 
 from kitty.conf.types import Definition, MultiOption, Option, ParserFuncType, unset
 from kitty.types import _T
 
 
-def chunks(lst: List[_T], n: int) -> Iterator[List[_T]]:
+def chunks(lst: list[_T], n: int) -> Iterator[list[_T]]:
     for i in range(0, len(lst), n):
         yield lst[i:i + n]
 
@@ -21,20 +22,20 @@ def atoi(text: str) -> str:
     return f'{int(text):08d}' if text.isdigit() else text
 
 
-def natural_keys(text: str) -> Tuple[str, ...]:
+def natural_keys(text: str) -> tuple[str, ...]:
     return tuple(atoi(c) for c in re.split(r'(\d+)', text))
 
 
-def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
-    class_lines: List[str] = []
-    tc_lines: List[str] = []
+def generate_class(defn: Definition, loc: str) -> tuple[str, str]:
+    class_lines: list[str] = []
+    tc_lines: list[str] = []
     a = class_lines.append
     t = tc_lines.append
     a('class Options:')
     t('class Parser:')
     choices = {}
-    imports: Set[Tuple[str, str]] = set()
-    tc_imports: Set[Tuple[str, str]] = set()
+    imports: set[tuple[str, str]] = set()
+    tc_imports: set[tuple[str, str]] = set()
     ki_imports: 're.Pattern[str]' = re.compile(r'\b((?:kittens|kitty).+?)[,\]]')
 
     def option_type_as_str(x: Any) -> str:
@@ -49,7 +50,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
             imports.add((x.__module__, x.__name__))
         return ans
 
-    def option_type_data(option: Union[Option, MultiOption]) -> Tuple[Callable[[Any], Any], str]:
+    def option_type_data(option: Option | MultiOption) -> tuple[Callable[[Any], Any], str]:
         func = option.parser_func
         if func.__module__ == 'builtins':
             return func, func.__name__
@@ -58,7 +59,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
         typ = option_type_as_str(rettype)
         if isinstance(option, MultiOption):
             typ = typ[typ.index('[') + 1:-1]
-            typ = typ.replace('Tuple', 'Dict', 1)
+            typ = typ.replace('tuple', 'dict', 1)
             kq = ki_imports.search(typ)
             if kq is not None:
                 kqi = kq.group(1)
@@ -69,18 +70,18 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     is_mutiple_vars = {}
     option_names = set()
     color_table = list(map(str, range(256)))
-    choice_dedup: Dict[str, str] = {}
-    choice_parser_dedup: Dict[str, str] = {}
+    choice_dedup: dict[str, str] = {}
+    choice_parser_dedup: dict[str, str] = {}
 
     def parser_function_declaration(option_name: str) -> None:
         t('')
-        t(f'    def {option_name}(self, val: str, ans: typing.Dict[str, typing.Any]) -> None:')
+        t(f'    def {option_name}(self, val: str, ans: dict[str, typing.Any]) -> None:')
 
     for option in sorted(defn.iter_all_options(), key=lambda a: natural_keys(a.name)):
         option_names.add(option.name)
         parser_function_declaration(option.name)
         if isinstance(option, MultiOption):
-            mval: Dict[str, Dict[str, Any]] = {'macos': {}, 'linux': {}, '': {}}
+            mval: dict[str, dict[str, Any]] = {'macos': {}, 'linux': {}, '': {}}
             func, typ = option_type_data(option)
             for val in option:
                 if val.add_to_default:
@@ -178,7 +179,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
         rettype = th['return']
         typ = option_type_as_str(rettype)
         typ = typ[typ.index('[') + 1:-1]
-        a(f'    {aname}: typing.List[{typ}] = []')
+        a(f'    {aname}: list[{typ}] = []')
         for imp in action.imports:
             resolve_import(imp)
         for fname, ftype in action.fields.items():
@@ -197,11 +198,11 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
             a('        ' + ', '.join(grp) + ',')
         a('    ))')
 
-    a('    config_paths: typing.Tuple[str, ...] = ()')
-    a('    all_config_paths: typing.Tuple[str, ...] = ()')
-    a('    config_overrides: typing.Tuple[str, ...] = ()')
+    a('    config_paths: tuple[str, ...] = ()')
+    a('    all_config_paths: tuple[str, ...] = ()')
+    a('    config_overrides: tuple[str, ...] = ()')
     a('')
-    a('    def __init__(self, options_dict: typing.Optional[typing.Dict[str, typing.Any]] = None) -> None:')
+    a('    def __init__(self, options_dict: dict[str, typing.Any] | None = None) -> None:')
     if defn.has_color_table:
         a('        self.color_table = array(self.color_table.typecode, self.color_table)')
     a('        if options_dict is not None:')
@@ -213,7 +214,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
 
     a('')
     a('    @property')
-    a('    def _fields(self) -> typing.Tuple[str, ...]:')
+    a('    def _fields(self) -> tuple[str, ...]:')
     a('        return option_names')
 
     a('')
@@ -232,7 +233,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     a('        return ans')
 
     a('')
-    a('    def _asdict(self) -> typing.Dict[str, typing.Any]:')
+    a('    def _asdict(self) -> dict[str, typing.Any]:')
     a('        return {k: self._copy_of_val(k) for k in self}')
 
     a('')
@@ -245,7 +246,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     a('        return ans')
 
     a('')
-    a('    def __getitem__(self, key: typing.Union[int, str]) -> typing.Any:')
+    a('    def __getitem__(self, key: int | str) -> typing.Any:')
     a('        k = option_names[key] if isinstance(key, int) else key')
     a('        try:')
     a('            return getattr(self, k)')
@@ -293,7 +294,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     a('')
     for aname, func in action_parsers.items():
         a(f'defaults.{aname} = [')
-        only: Dict[str, List[Tuple[str, Callable[..., Any]]]] = {}
+        only: dict[str, list[tuple[str, Callable[..., Any]]]] = {}
         for sc in defn.iter_all_maps(aname):
             if not sc.add_to_default:
                 continue
@@ -318,7 +319,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
 
     t('')
     t('')
-    t('def create_result_dict() -> typing.Dict[str, typing.Any]:')
+    t('def create_result_dict() -> dict[str, typing.Any]:')
     t('    return {')
     for oname in is_mutiple_vars:
         t(f'        {oname!r}: {{}},')
@@ -328,10 +329,10 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
 
     t('')
     t('')
-    t(f'actions: typing.FrozenSet[str] = frozenset({tuple(defn.actions)!r})')
+    t(f'actions: frozenset[str] = frozenset({tuple(defn.actions)!r})')
     t('')
     t('')
-    t('def merge_result_dicts(defaults: typing.Dict[str, typing.Any], vals: typing.Dict[str, typing.Any]) -> typing.Dict[str, typing.Any]:')
+    t('def merge_result_dicts(defaults: dict[str, typing.Any], vals: dict[str, typing.Any]) -> dict[str, typing.Any]:')
     t('    ans = {}')
     t('    for k, v in defaults.items():')
     t('        if isinstance(v, dict):')
@@ -348,7 +349,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     t('parser = Parser()')
     t('')
     t('')
-    t('def parse_conf_item(key: str, val: str, ans: typing.Dict[str, typing.Any]) -> bool:')
+    t('def parse_conf_item(key: str, val: str, ans: dict[str, typing.Any]) -> bool:')
     t('    func = getattr(parser, key, None)')
     t('    if func is not None:')
     t('        func(val, ans)')
@@ -358,11 +359,12 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     preamble = ['# generated by gen-config.py DO NOT edit', '']
     a = preamble.append
 
-    def output_imports(imports: Set[Tuple[str, str]], add_module_imports: bool = True) -> None:
+    def output_imports(imports: set[tuple[str, str]], add_module_imports: bool = True) -> None:
         a('# isort: skip_file')
         a('import typing')
+        a('import collections.abc  # noqa: F401, RUF100')
         seen_mods = {'typing'}
-        mmap: Dict[str, List[str]] = {}
+        mmap: dict[str, list[str]] = {}
         for mod, name in imports:
             mmap.setdefault(mod, []).append(name)
         for mod in sorted(mmap):
@@ -401,8 +403,8 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     return class_def, '\n'.join(preamble + ['', ''] + tc_lines)
 
 
-def generate_c_conversion(loc: str, ctypes: List[Union[Option, MultiOption]]) -> str:
-    lines: List[str] = []
+def generate_c_conversion(loc: str, ctypes: list[Option | MultiOption]) -> str:
+    lines: list[str] = []
     basic_converters = {
         'int': 'PyLong_AsLong', 'uint': 'PyLong_AsUnsignedLong', 'bool': 'PyObject_IsTrue',
         'float': 'PyFloat_AsFloat', 'double': 'PyFloat_AsDouble', 'percent': 'percent',
@@ -462,7 +464,7 @@ def write_output(loc: str, defn: Definition, extra_after_type_defn: str = '') ->
             f.write(f'{c}\n')
 
 
-def go_type_data(parser_func: ParserFuncType, ctype: str, is_multiple: bool = False) -> Tuple[str, str]:
+def go_type_data(parser_func: ParserFuncType, ctype: str, is_multiple: bool = False) -> tuple[str, str]:
     if ctype:
         if ctype == 'string':
             if is_multiple:
