@@ -6,11 +6,12 @@
  */
 
 #include "data-types.h"
+#include "monotonic.h"
 
 #define CMD_BUF_SZ 2048
 
 
-static inline bool
+static bool
 append_buf(char buf[CMD_BUF_SZ], size_t *pos, PyObject *ans) {
     if (*pos) {
         PyObject *bytes = PyBytes_FromStringAndSize(buf, *pos);
@@ -23,7 +24,7 @@ append_buf(char buf[CMD_BUF_SZ], size_t *pos, PyObject *ans) {
     return true;
 }
 
-static inline bool
+static bool
 add_char(char buf[CMD_BUF_SZ], size_t *pos, char ch, PyObject *ans) {
     if (*pos >= CMD_BUF_SZ) {
         if (!append_buf(buf, pos, ans)) return false;
@@ -33,14 +34,14 @@ add_char(char buf[CMD_BUF_SZ], size_t *pos, char ch, PyObject *ans) {
     return true;
 }
 
-static inline bool
-read_response(int fd, double timeout, PyObject *ans) {
+static bool
+read_response(int fd, monotonic_t timeout, PyObject *ans) {
     static char buf[CMD_BUF_SZ];
     size_t pos = 0;
     enum ReadState {START, STARTING_ESC, P, AT, K, I, T, T2, Y, HYPHEN, C, M, BODY, TRAILING_ESC};
     enum ReadState state = START;
     char ch;
-    double end_time = monotonic() + timeout;
+    monotonic_t end_time = monotonic() + timeout;
     while(monotonic() <= end_time) {
         ssize_t len = read(fd, &ch, 1);
         if (len == 0) continue;
@@ -53,6 +54,7 @@ read_response(int fd, double timeout, PyObject *ans) {
         switch(state) {
             case START:
                 if (ch == 0x1b) state = STARTING_ESC;
+                if (ch == 0x03) { PyErr_SetString(PyExc_KeyboardInterrupt, "User pressed Ctrl+C"); return false; }
                 break;
 #define CASE(curr, q, next) case curr: state = ch == q ? next : START; break;
             CASE(STARTING_ESC, 'P', P);
@@ -94,7 +96,7 @@ read_command_response(PyObject *self UNUSED, PyObject *args) {
     int fd;
     PyObject *ans;
     if (!PyArg_ParseTuple(args, "idO!", &fd, &timeout, &PyList_Type, &ans)) return NULL;
-    if (!read_response(fd, timeout, ans)) return NULL;
+    if (!read_response(fd, s_double_to_monotonic_t(timeout), ans)) return NULL;
     Py_RETURN_NONE;
 }
 
