@@ -7,7 +7,6 @@ import shutil
 import sys
 from collections.abc import Generator, Sequence
 from contextlib import contextmanager, suppress
-from typing import Optional
 
 from .borders import load_borders_program
 from .boss import Boss
@@ -29,6 +28,7 @@ from .constants import (
     kitty_exe,
     logo_png_file,
     running_in_kitty,
+    supports_window_occlusion,
     website_url,
 )
 from .fast_data_types import (
@@ -41,11 +41,11 @@ from .fast_data_types import (
     glfw_terminate,
     load_png_data,
     mask_kitty_signals_process_wide,
+    run_at_exit_cleanup_functions,
     set_custom_cursor,
     set_default_window_icon,
     set_options,
 )
-from .fonts.box_drawing import set_scale
 from .fonts.render import dump_font_debug, set_font_family
 from .options.types import Options
 from .options.utils import DELETE_ENV_VAR
@@ -89,9 +89,11 @@ def load_all_shaders(semi_transparent: bool = False) -> None:
         raise SystemExit(err)
 
 
-def init_glfw_module(glfw_module: str, debug_keyboard: bool = False, debug_rendering: bool = False, wayland_enable_ime: bool = True) -> None:
-    if not glfw_init(glfw_path(glfw_module), edge_spacing, debug_keyboard, debug_rendering, wayland_enable_ime):
+def init_glfw_module(glfw_module: str = 'wayland', debug_keyboard: bool = False, debug_rendering: bool = False, wayland_enable_ime: bool = True) -> None:
+    ok, swo = glfw_init(glfw_path(glfw_module), edge_spacing, debug_keyboard, debug_rendering, wayland_enable_ime)
+    if not ok:
         raise SystemExit('GLFW initialization failed')
+    supports_window_occlusion(swo)
 
 
 def init_glfw(opts: Options, debug_keyboard: bool = False, debug_rendering: bool = False) -> str:
@@ -102,7 +104,7 @@ def init_glfw(opts: Options, debug_keyboard: bool = False, debug_rendering: bool
 
 def get_macos_shortcut_for(
     func_map: dict[tuple[str, ...], list[SingleKey]], defn: str = 'new_os_window', lookup_name: str = ''
-) -> Optional[SingleKey]:
+) -> SingleKey | None:
     # for maximum robustness we should use opts.alias_map to resolve
     # aliases however this requires parsing everything on startup which could
     # be potentially slow. Lets just hope the user doesn't alias these
@@ -154,17 +156,19 @@ def get_icon128_path(base_path: str) -> str:
     return f'{path}-128{ext}'
 
 
-def set_x11_window_icon() -> None:
+def set_window_icon() -> None:
     custom_icon_path = get_custom_window_icon()[1]
+    is_x11 = not is_macos and not is_wayland()
     try:
         if custom_icon_path is not None:
             custom_icon128_path = get_icon128_path(custom_icon_path)
-            if safe_mtime(custom_icon128_path) is None:
-                set_default_window_icon(custom_icon_path)
-            else:
+            if is_x11 and safe_mtime(custom_icon128_path) is not None:
                 set_default_window_icon(custom_icon128_path)
+            else:
+                set_default_window_icon(custom_icon_path)
         else:
-            set_default_window_icon(get_icon128_path(logo_png_file))
+            if is_x11:
+                set_default_window_icon(get_icon128_path(logo_png_file))
     except ValueError as err:
         log_error(err)
 
@@ -215,12 +219,12 @@ def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = (),
         set_macos_app_custom_icon()
     else:
         global_shortcuts = {}
-        if not is_wayland():  # no window icons on wayland
-            set_x11_window_icon()
+        set_window_icon()
 
     with cached_values_for(run_app.cached_values_name) as cached_values:
         startup_sessions = tuple(create_sessions(opts, args, default_session=opts.startup_session))
         wincls = (startup_sessions[0].os_window_class if startup_sessions else '') or args.cls or appname
+        winname = (startup_sessions[0].os_window_name if startup_sessions else '') or args.name or wincls or appname
         window_state = (args.start_as if args.start_as and args.start_as != 'normal' else None) or (
             getattr(startup_sessions[0], 'os_window_state', None) if startup_sessions else None
         )
@@ -229,7 +233,7 @@ def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = (),
             window_id = create_os_window(
                     run_app.initial_window_size_func(get_os_window_sizing_data(opts, startup_sessions[0] if startup_sessions else None), cached_values),
                     pre_show_callback,
-                    args.title or appname, args.name or args.cls or appname,
+                    args.title or appname, winname,
                     wincls, wstate, load_all_shaders, disallow_override_title=bool(args.title), layer_shell_config=run_app.layer_shell_config)
         boss = Boss(opts, args, cached_values, global_shortcuts, talk_fd)
         boss.start(window_id, startup_sessions)
@@ -249,11 +253,10 @@ class AppRunner:
     def __init__(self) -> None:
         self.cached_values_name = 'main'
         self.first_window_callback = lambda window_handle: None
-        self.layer_shell_config: Optional[LayerShellConfig] = None
+        self.layer_shell_config: LayerShellConfig | None = None
         self.initial_window_size_func = initial_window_size_func
 
     def __call__(self, opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = (), talk_fd: int = -1) -> None:
-        set_scale(opts.box_drawing_scale)
         set_options(opts, is_wayland(), args.debug_rendering, args.debug_font_fallback)
         if theme_colors.refresh():
             theme_colors.patch_opts(opts, args.debug_rendering)
@@ -473,9 +476,9 @@ def _main() -> None:
     if not cwd_ok:
         os.chdir(os.path.expanduser('~'))
     if getattr(sys, 'cmdline_args_for_open', False):
-        usage: Optional[str] = 'file_or_url ...'
-        appname: Optional[str] = 'kitty +open'
-        msg: Optional[str] = (
+        usage: str | None = 'file_or_url ...'
+        appname: str | None = 'kitty +open'
+        msg: str | None = (
             'Run kitty and open the specified files or URLs in it, using launch-actions.conf. For details'
             ' see https://sw.kovidgoyal.net/kitty/open_actions/#scripting-the-opening-of-files-with-kitty-on-macos'
             '\n\nAll the normal kitty options can be used.')
@@ -540,3 +543,7 @@ def main() -> None:
         tb = traceback.format_exc()
         log_error(tb)
         raise SystemExit(1)
+    finally:
+        # we cant rely on this running during module unloading of fast_data_types as Python fails
+        # to unload the module, due to reference cycles, I am guessing.
+        run_at_exit_cleanup_functions()
