@@ -7,7 +7,7 @@ import posixpath
 import shlex
 from collections.abc import Iterable, Iterator
 from contextlib import suppress
-from typing import Any, NamedTuple, Optional, cast
+from typing import Any, NamedTuple, cast
 from urllib.parse import ParseResult, unquote, urlparse
 
 from .conf.utils import KeyAction, to_cmdline_implementation
@@ -240,25 +240,27 @@ action show_kitty_doc $URL_PATH
 @run_once
 def default_launch_actions() -> tuple[OpenAction, ...]:
     return tuple(parse('''\
-# Open script files
+# Open script files. Change confirm-always to confirm-never or confirm-if-needed to
+# disable confirmation for all or executable files respectively.
 protocol file
 ext sh,command,tool
-action launch --hold --type=os-window kitty +shebang $FILE_PATH $SHELL
+action launch --hold --type=os-window kitten __shebang__ confirm-always $FILE_PATH $SHELL
 
 # Open shell specific script files
 protocol file
 ext fish,bash,zsh
-action launch --hold --type=os-window kitty +shebang $FILE_PATH __ext__
+action launch --hold --type=os-window kitten __shebang__ confirm-always $FILE_PATH __ext__
 
 # Open directories
 protocol file
 mime inode/directory
 action launch --type=os-window --cwd -- $FILE_PATH
 
-# Open executable file
+# Open executable file. Remove kitten __confirm_and_run_exe__ to execute
+# without confirmation.
 protocol file
 mime inode/executable,application/vnd.microsoft.portable-executable
-action launch --hold --type=os-window -- $FILE_PATH
+action launch --hold --type=os-window -- kitten __confirm_and_run_exe__ $FILE_PATH
 
 # Open text files without fragments in the editor
 protocol file
@@ -276,7 +278,7 @@ action launch --type=os-window ssh -- $URL
 '''.splitlines()))
 
 
-def actions_for_url(url: str, actions_spec: Optional[str] = None) -> Iterator[KeyAction]:
+def actions_for_url(url: str, actions_spec: str | None = None) -> Iterator[KeyAction]:
     if actions_spec is None:
         actions = load_open_actions()
     else:
