@@ -58,6 +58,23 @@ window_title_in(PyObject *title_in) {
     return ALL;
 }
 
+static inline unsigned
+undercurl_style(PyObject *x) {
+    RAII_PyObject(thick, PyUnicode_FromString("thick"));
+    RAII_PyObject(dense, PyUnicode_FromString("dense"));
+    unsigned ans = 0;
+    int ret;
+    switch ((ret = PyUnicode_Find(x, dense, 0, PyUnicode_GET_LENGTH(x), 1))) {
+        case -2: PyErr_Clear(); case -1: break;
+        default: ans |= 1;
+    }
+    switch ((ret = PyUnicode_Find(x, thick, 0, PyUnicode_GET_LENGTH(x), 1))) {
+        case -2: PyErr_Clear(); case -1: break;
+        default: ans |= 2;
+    }
+    return ans;
+}
+
 static inline UnderlineHyperlinks
 underline_hyperlinks(PyObject *x) {
     const char *in = PyUnicode_AsUTF8(x);
@@ -283,6 +300,12 @@ pointer_shape(PyObject *shape_name) {
     return TEXT_POINTER;
 }
 
+static inline void
+dragging_pointer_shape(PyObject *parts, Options *opts) {
+    opts->pointer_shape_when_dragging = pointer_shape(PyTuple_GET_ITEM(parts, 0));
+    opts->pointer_shape_when_dragging_rectangle = pointer_shape(PyTuple_GET_ITEM(parts, 1));
+}
+
 static inline int
 macos_colorspace(PyObject *csname) {
     if (PyUnicode_CompareWithASCIIString(csname, "srgb") == 0) return 1;
@@ -363,6 +386,23 @@ menu_map(PyObject *entry_dict, Options *opts) {
     }
 }
 
+static inline void
+underline_exclusion(PyObject *val, Options *opts) {
+    if (!PyTuple_Check(val)) { PyErr_SetString(PyExc_TypeError, "underline_exclusion must be a tuple"); return; }
+    opts->underline_exclusion.thickness = PyFloat_AsFloat(PyTuple_GET_ITEM(val, 0));
+    if (!PyUnicode_GET_LENGTH(PyTuple_GET_ITEM(val, 1))) opts->underline_exclusion.unit = 0;
+    else if (PyUnicode_CompareWithASCIIString(PyTuple_GET_ITEM(val, 1), "px")) opts->underline_exclusion.unit = 1;
+    else if (PyUnicode_CompareWithASCIIString(PyTuple_GET_ITEM(val, 1), "pt")) opts->underline_exclusion.unit = 2;
+    else opts->underline_exclusion.unit = 0;
+}
+
+static inline void
+box_drawing_scale(PyObject *val, Options *opts) {
+    for (unsigned i = 0; i < MIN(arraysz(opts->box_drawing_scale), (size_t)PyTuple_GET_SIZE(val)); i++) {
+        opts->box_drawing_scale[i] = PyFloat_AsFloat(PyTuple_GET_ITEM(val, i));
+    }
+}
+
 static inline void
 text_composition_strategy(PyObject *val, Options *opts) {
     if (!PyUnicode_Check(val)) { PyErr_SetString(PyExc_TypeError, "text_rendering_strategy must be a string"); return; }
@@ -440,7 +480,7 @@ tab_bar_margin_height(PyObject *val, Options *opts) {
     opts->tab_bar_margin_height.inner = PyFloat_AsDouble(PyTuple_GET_ITEM(val, 1));
 }
 
-static void
+static inline void
 window_logo_scale(PyObject *src, Options *opts) {
     opts->window_logo_scale.width = PyFloat_AsFloat(PyTuple_GET_ITEM(src, 0));
     opts->window_logo_scale.height = PyFloat_AsFloat(PyTuple_GET_ITEM(src, 1));
