@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # License: GPLv3 Copyright: 2020, Kovid Goyal <kovid at kovidgoyal.net>
 
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
 from .base import ArgsType, Boss, PayloadGetType, PayloadType, RCOptions, RemoteCommand, ResponseType, Window
 
@@ -42,7 +42,7 @@ the font size for any newly created OS Windows in the future.
         inc = fs[0] if fs and fs[0] in '+-' else None
         return {'size': abs(float(fs)), 'all': opts.all, 'increment_op': inc}
 
-    def response_from_kitty(self, boss: Boss, window: Optional[Window], payload_get: PayloadGetType) -> ResponseType:
+    def response_from_kitty(self, boss: Boss, window: Window | None, payload_get: PayloadGetType) -> ResponseType:
         boss.change_font_size(
             payload_get('all'),
             payload_get('increment_op'), payload_get('size') or 0)
