@@ -1,17 +1,21 @@
 #!/usr/bin/env python
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
+import json
 import os
 import sys
 import tempfile
 
+from kitty.constants import read_kitty_resource
 from kitty.fast_data_types import (
     Color,
     HistoryBuf,
     LineBuf,
+    char_props_for,
     expand_ansi_c_escapes,
     parse_input_from_terminal,
     replace_c0_codes_except_nl_space_tab,
+    split_into_graphemes,
     strip_csi,
     truncate_point_for_length,
     wcswidth,
@@ -240,19 +244,6 @@ class TestDataTypes(BaseTest):
         l3 = lb.line(0)
         l3.set_text(t, 0, len(t), C())
         self.ae(t, str(l3))
-        l3.right_shift(4, 2)
-        self.ae('0123454567', str(l3))
-        l3.set_text(t, 0, len(t), C())
-        l3.right_shift(0, 0)
-        self.ae(t, str(l3))
-        l3.right_shift(0, 1)
-        self.ae(str(l3), '0' + t[:-1])
-        l3.set_text(t, 0, len(t), C())
-        l3.left_shift(0, 2)
-        self.ae(str(l3), t[2:] + '89')
-        l3.set_text(t, 0, len(t), C())
-        l3.left_shift(7, 3)
-        self.ae(str(l3), t)
 
         l3.set_text(t, 0, len(t), C())
         q = C()
@@ -327,28 +318,25 @@ class TestDataTypes(BaseTest):
         self.ae(l4.url_end_at(0), len(l4) - 2)
         self.ae(l4.url_end_at(0, 0, True), len(l4) - 1)
 
-    def rewrap(self, lb, lb2):
-        hb = HistoryBuf(lb2.ynum, lb2.xnum)
-        cy = lb.rewrap(lb2, hb)
-        return hb, cy[1]
+    def rewrap(self, lb, lines, columns):
+        return lb.rewrap(lines, columns)
 
     def test_rewrap_simple(self):
         ' Same width buffers '
         lb = filled_line_buf(5, 5)
         lb2 = LineBuf(lb.ynum, lb.xnum)
-        self.rewrap(lb, lb2)
+        lb2 = self.rewrap(lb, lb.ynum, lb.xnum)[0]
         for i in range(lb.ynum):
             self.ae(lb2.line(i), lb.line(i))
-        lb2 = LineBuf(8, 5)
-        cy = self.rewrap(lb, lb2)[1]
+        lb2, _, cy = self.rewrap(lb, 8, 5)
         self.ae(cy, 5)
         for i in range(lb.ynum):
-            self.ae(lb2.line(i), lb.line(i))
+            self.ae(lb2.line(i), lb.line(i), i)
         empty = LineBuf(1, lb2.xnum)
         for i in range(lb.ynum, lb2.ynum):
             self.ae(str(lb2.line(i)), str(empty.line(0)))
         lb2 = LineBuf(3, 5)
-        cy = self.rewrap(lb, lb2)[1]
+        lb2, _, cy = self.rewrap(lb, 3, 5)
         self.ae(cy, 3)
         for i in range(lb2.ynum):
             self.ae(lb2.line(i), lb.line(i + 2))
@@ -361,8 +349,7 @@ class TestDataTypes(BaseTest):
             self.ae(l0, str(l2))
 
     def line_comparison_rewrap(self, lb, *lines):
-        lb2 = LineBuf(len(lines), max(map(len, lines)))
-        self.rewrap(lb, lb2)
+        lb2 = self.rewrap(lb, len(lines), max(map(len, lines)))[0]
         self.line_comparison(lb2, *lines)
         return lb2
 
@@ -499,35 +486,31 @@ class TestDataTypes(BaseTest):
             self.ae(str(hb.line(i)).rstrip(), str(3000 - 1 - i))
 
         # rewrap
+        def as_ansi(hb):
+            lines = []
+            hb.as_ansi(lines.append)
+            return ''.join(lines)
         hb = filled_history_buf(5, 5)
-        hb2 = HistoryBuf(hb.ynum, hb.xnum)
-        hb.rewrap(hb2)
+        for i in range(hb.ynum):
+            hb.line(i).set_wrapped_flag(True)
+        before = as_ansi(hb)
+        hb2 = hb.rewrap(10)
+        self.ae(before, as_ansi(hb2).rstrip())
+
+        hb = filled_history_buf(5, 5)
+        hb2 = hb.rewrap(hb.xnum)
         for i in range(hb.ynum):
             self.ae(hb2.line(i), hb.line(i))
-        hb2 = HistoryBuf(8, 5)
-        hb.rewrap(hb2)
-        for i in range(hb.ynum):
-            self.ae(hb2.line(i), hb.line(i))
-        for i in range(hb.ynum, hb2.ynum):
-            with self.assertRaises(IndexError):
-                hb2.line(i)
-        hb2 = HistoryBuf(3, 5)
-        hb.rewrap(hb2)
-        for i in range(hb2.ynum):
-            self.ae(hb2.line(i), hb.line(i))
-        self.ae(hb2.dirty_lines(), list(range(hb2.ynum)))
         hb = filled_history_buf(5, 5)
-        hb2 = HistoryBuf(hb.ynum, hb.xnum * 2)
-        hb.rewrap(hb2)
+        hb2 = hb.rewrap(hb.xnum * 2)
         hb3 = HistoryBuf(hb.ynum, hb.xnum)
-        hb2.rewrap(hb3)
+        hb3 = hb2.rewrap(hb.xnum)
         for i in range(hb.ynum):
             self.ae(hb.line(i), hb3.line(i))
 
         hb2 = HistoryBuf(hb.ynum, hb.xnum)
-        large_hb.rewrap(hb2)
-        hb2 = HistoryBuf(large_hb.ynum, large_hb.xnum)
-        large_hb.rewrap(hb2)
+        hb2 = large_hb.rewrap(hb.xnum)
+        hb2.rewrap(large_hb.xnum)
 
     def test_ansi_repr(self):
         lb = filled_line_buf()
@@ -652,3 +635,11 @@ class TestDataTypes(BaseTest):
         }.items():
             actual = tuple(shlex_split_with_positions(q, True))
             self.ae(expected, actual, f'Failed for text: {q!r}')
+
+    def test_split_into_graphemes(self):
+        self.assertEqual(char_props_for('\ue000')['category'], 'Co')
+        self.ae(split_into_graphemes('ab'), ['a', 'b'])
+        for i, test in enumerate(json.loads(read_kitty_resource('GraphemeBreakTest.json', __name__.rpartition('.')[0]))):
+            expected = test['data']
+            actual = split_into_graphemes(''.join(expected))
+            self.ae(expected, actual, f'Test #{i} failed: {test["comment"]}')
