@@ -78,6 +78,10 @@ func SetReadTimeout(d time.Duration) TermiosOperation {
 
 var SetBlockingRead TermiosOperation = SetReadTimeout(0)
 
+var SetNoCanonical TermiosOperation = func(t *unix.Termios) {
+	t.Lflag &^= unix.ICANON
+}
+
 var SetRaw TermiosOperation = func(t *unix.Termios) {
 	// This attempts to replicate the behaviour documented for cfmakeraw in
 	// the termios(3) manpage, as Go doesn't wrap cfmakeraw probably because its not in POSIX
@@ -328,14 +332,11 @@ func (self *Term) WriteString(b string) (int, error) {
 }
 
 func (self *Term) DebugPrintln(a ...any) {
-	msg := []byte(fmt.Sprintln(a...))
+	msg := fmt.Appendln(nil, a...)
 	const limit = 2048
 	encoded := make([]byte, limit*2)
 	for i := 0; i < len(msg); i += limit {
-		end := i + limit
-		if end > len(msg) {
-			end = len(msg)
-		}
+		end := min(i+limit, len(msg))
 		chunk := msg[i:end]
 		encoded = encoded[:cap(encoded)]
 		base64.StdEncoding.Encode(encoded, chunk)
@@ -383,3 +384,21 @@ func DebugPrintln(a ...any) {
 		term.DebugPrintln(a...)
 	}
 }
+
+func ReadSingleByteFromTerminal() (b byte, err error) {
+	term, err := OpenControllingTerm(SetBlockingRead, SetNoCanonical)
+	if err != nil {
+		return 0, err
+	}
+	defer term.Close()
+	ans := []byte{b}
+	for {
+		n, err := term.Read(ans)
+		if err != nil {
+			return 0, err
+		}
+		if n > 0 {
+			return ans[0], nil
+		}
+	}
+}
